/*
    Zelda Battle Of Hyrule
    Copyright (C) 2009  Vincent Bourgogne
    
    Origine des sources :
    Zelda Return of the Hylian
    Copyright (C) 2005-2008  Vincent Jouillat

    Please send bugreports with examples or suggestions to www.zeldaroth.fr

*/

#include "Joueur.h"
#include "Monde.h"
#include "Menu.h"
#include "Texte.h"
#include "Projectile.h"
#include "Jeu.h"
#include "Keyboard.h"
#include "Partie.h"
#include "Message.h"

#include <cstdlib>
#include <iostream>

Jeu::Jeu(Audio* audio) : gpAudio(audio), gpGenerique(0), gpKeyboard(0),
zone(21), stop(false), transition(false),
menu(false), text(false), vueHorz(0), vueVert(0), fct()  {
    phg[0] = 0;
    phg[1] = 0;
    
   
    imageObjets = IMG_Load("data/images/objet/objets.png");
    SDL_SetColorKey(imageObjets,SDL_SRCCOLORKEY,SDL_MapRGB(imageObjets->format,0,0,255));
    
    gpProjectile = NULL; gpObjet = NULL; gpSnipe = NULL; 
    gpJoueur = NULL; gpMonde = NULL; gpMenu = NULL;
    gpStatut = NULL;
    
    gpTexte = new Texte(this);
    
    gpPartie = new Partie(this);
    
    gpJoueur = new Joueur(this);
    
    gpMessage = NULL;
    
    //lecture de Joueur.txt
    std::string result = "data/info/Joueur.txt";
    std::string tmpPseudo = "Pseudo";
    int tmpCouleur = 1;
    std::ifstream fileJ(result.c_str());
    if (!fileJ)
    {
        std::cerr << "Impossible d'ouvrir le fichier Joueur.dat" << std::endl;
    }
    else
    {
        fileJ >> tmpPseudo;
        fileJ >> tmpCouleur;
    }
    fileJ.close();
    gpJoueur->setPseudo(tmpPseudo); 
    gpJoueur->setCouleur(tmpCouleur);
    
    
    //lecture de IP.txt
    result = "data/info/IP.txt";
    std::ifstream fileC(result.c_str());
    gpIP[0] = 127;
    gpIP[1] = 0;
    gpIP[2] = 0;
    gpIP[3] = 1;
    if (!fileC)
    {
        std::cerr << "Impossible d'ouvrir le fichier IP.txt" << std::endl;
    }
    else
    {
        for(int i = 0; i<4; i++) fileC >> gpIP[i];
    }
    fileC.close();

    //lecture de Ports.txt
    result = "data/info/Ports.txt";
    std::ifstream fileP(result.c_str());
    portTCP = 23734;
    portUDP = 23735;
    if (!fileP)
    {
        std::cerr << "Impossible d'ouvrir le fichier Ports.txt" << std::endl;
    }
    else
    {
        fileP >> portTCP;
        fileP >> portUDP;
    }
    fileP.close();

    
}

Jeu::~Jeu() {
    delete gpMonde;
    delete gpJoueur;
    delete gpMenu;
    delete gpTexte;
    delete gpStatut;
    delete gpProjectile;
    delete gpObjet;
    delete gpSnipe;
    delete gpMessage;
    delete gpPartie;
}

void Jeu::retourMenu()
{
}

bool Jeu::init(int Type) {
    
    gpMessage = new Message(this, 0, 0, 0);
    
    int ConnexionOK;
    
    gpType = Type;
    if (gpType==1)
    { 
      //le serveur commence directement dans l'arene
      zone=gpPartie->getAreneToZone();             
      lastZone=zone;
      int posX, posY;
      gpPartie->positionRevie(posX,posY);
      gpJoueur->setX(posX);
      gpJoueur->setY(posY);
      gpJoueur->setNumero(0);
      gpServeur = new Serveur(this);
      ConnexionOK = gpServeur->init();
      
      if (ConnexionOK!=0)//si problme de lancement du serveur
        delete gpServeur;
      
    }
    else
    {
      zone=21;
      lastZone=zone;
      gpClient = new Client(this);
      
      ConnexionOK = gpClient->init();
      
      if (ConnexionOK!=0)//si problme de lancement du client
        delete gpClient;
    }
     
     
    if (ConnexionOK == 0)
    {
        nbAutreJoueur = gpPartie->getNbJoueur();
        gpAutreJoueur = new Joueur*[nbAutreJoueur];
        gpAutreJoueur[gpJoueur->getNumero()] = gpJoueur;
        
        for(int i=0; i<nbAutreJoueur; i++)
        {
            if(i != gpJoueur->getNumero())
            {
               gpAutreJoueur[i] = new Joueur(this);
               gpAutreJoueur[i]->setNumero(i);
               gpAutreJoueur[i]->nonVisible();
            }
            gpPartie->ModifierJoueur(i);
        }
        
        gpProjectile = new Projectile(this, 0, N, 0, 0, 0);
        gpObjet = new Objet(this, 0, 0, 0, 0);
        gpSnipe = new Snipe(this, 0, 0, 0, 0, 0);
        gpMonde = new Monde(this);
        gpMenu = new Menu(this);
        gpStatut = new Statut(this);
        gpAudio->playMusic(zone);
        
        if (gpType!=1)
        {
           gpClient->lanceThread();
        }
    }
    
    return (ConnexionOK==0);
    
}

void Jeu::setKeyboard(Keyboard* kb) {
    gpKeyboard = kb;
}

void Jeu::setGenerique(Generique* gen) {
    gpGenerique = gen;
}

void Jeu::videListes() {
    delete gpProjectile;
    gpProjectile = new Projectile(this, 0, N, 0, 0, 0);
    delete gpObjet;
    gpObjet = new Objet(this, 0, 0, 0, 0);
    delete gpSnipe;
    gpSnipe = new Snipe(this, 0, 0, 0, 0, 0);

}

void Jeu::scrolling() {
    phg[0] = gpJoueur->getX() - 320/2 + 8 + vueHorz;
    phg[1] = gpJoueur->getY() - 240/2 + 16 + vueVert;
    if (phg[0] < gpMonde->regionValue(0)) phg[0] = gpMonde->regionValue(0);
    if (phg[1] < gpMonde->regionValue(1)) phg[1] = gpMonde->regionValue(1);
    if (phg[0] > gpMonde->regionValue(2) - 320) phg[0] = gpMonde->regionValue(2) - 320;
    if (phg[1] > gpMonde->regionValue(3) - 240) phg[1] = gpMonde->regionValue(3) - 240;
}

void Jeu::replace() {
   
    gpObjet->revie();
    delete gpSnipe;
    gpSnipe = new Snipe(this, 0, 0, 0, 0, 0);
    delete gpProjectile;
    gpProjectile = new Projectile(this, 0, N, 0, 0, 0);
    gpMonde->replace();
}


void Jeu::draw(SDL_Surface* screen) {
    
    int i;
    
    if (!transition) scrolling();
    
    int ordreJoueur[nbAutreJoueur];
    int planJoueur[nbAutreJoueur];
    int tailleTab=0;
    gpJoueur->setZone(zone);
    
    if(zone!=lastZone)
    {
        for (i=0;i <nbAutreJoueur; i++)
        {
           if(!gpAutreJoueur[i]->estInvisible() && gpAutreJoueur[i]->getZone()==zone)
           {
             gpAutreJoueur[i]->setFantome(0);
           }
        }
        lastZone=zone;
    }
    
       
    for (i=0;i <nbAutreJoueur; i++)
    {
       if(!gpAutreJoueur[i]->estInvisible() && gpAutreJoueur[i]->getZone()==zone)
       {
         ordreJoueur[tailleTab]=i;
         planJoueur[tailleTab]=gpAutreJoueur[i]->getY(); 
         
         tailleTab++;
       }
    }
        
    fct.tri_rapide2T(planJoueur, ordreJoueur, tailleTab);
    
    gpMonde->drawSol(screen, phg);
    
    
    if (!transition) {
        gpObjet->draw(screen);
        gpSnipe->draw(screen);
        gpProjectile->draw(screen);
    }
    
    if (gpType==1)
    {
      gpServeur->imitClient();
      gpServeur->testProjectile();
    }
   
    
    if(zone==21) 
    {
        gpTexte->affiche(screen, "Bienvenue", 50, 20);
        gpTexte->affiche(screen, "Arene : "+gpPartie->getAreneName(), 50, 35);
        gpTexte->affiche(screen, "Type de jeu : "+gpPartie->getTypeJeuName(), 50, 50);
        gpTexte->affiche(screen, "Vous tes : "+gpJoueur->getNomComplet(), 50, 65);
    }
    
    if(zone!=21) //si on est pas dans la salle avant l'arene
    {
        for (i=0;i <tailleTab; i++)
        {
            gpAutreJoueur[ordreJoueur[i]]->draw(screen);
        } 
        
        if (gpType==1)
        {
            gpServeur->getProjectile()->draw(screen);
        }
        else
        {
            gpClient->getProjectile()->drawPrClient(screen);
        }
    }
    else 
         gpJoueur->draw(screen);
         
    gpMonde->drawAir(screen, phg);
    
    if (transition) gpMonde->defilTransit();
    
    gpStatut->draw(screen);
    gpMessage->draw(screen,0);
    
    if (menu || gpMenu->getVal()) gpMenu->draw(screen);
    if (text && !transition) gpTexte->draw(screen);
}

Joueur* Jeu::getJoueur() {
    return gpJoueur;
}

Joueur* Jeu::getAutreJoueur(int i) {
    return gpAutreJoueur[i];
}

Monde* Jeu::getMonde() {
    return gpMonde;
}

Audio* Jeu::getAudio() {
    return gpAudio;
}

Texte* Jeu::getTexte() {
    return gpTexte;
}

Statut* Jeu::getStatut() {
    return gpStatut;
}

Generique* Jeu::getGenerique() {
    return gpGenerique;
}

Keyboard* Jeu::getKeyboard() {
    return gpKeyboard;
}

Partie* Jeu::getPartie() {
    return gpPartie;
}


bool Jeu::estServeur()
{
    return (gpType==1); 
}


int Jeu::getIP(int i) {
    return gpIP[i];
}

int Jeu::getPortTCP() {
    return portTCP;
}

int Jeu::getPortUDP() {
    return portUDP;
}


string Jeu::getIPName() {
    //string im;
    std::ostringstream out;
    out << gpIP[0];
    out << ".";
    out << gpIP[1];
    out << ".";
    out << gpIP[2];
    out << ".";
    out << gpIP[3];
    
    return out.str();
}
        
void Jeu::setIP(int a, int i) {
    if (gpIP[i] == 0) gpIP[i] = 256;
    gpIP[i] = ((gpIP[i]+a)%256);
}


int Jeu::getNbAutreJoueur() {
    return nbAutreJoueur;
}

void Jeu::setMessage(int T, int J1, int J2) {
    gpMessage->ajout(new Message(this,T,J1,J2));
    
}

Message* Jeu::getMessage() {
    return gpMessage;
}

int Jeu::dir2Int(Direction d)
{
    int a;
    
    switch(d){
       case N : a=0;break;
       case S : a=1;break;
       case O : a=2;break;
       case E : a=3;break;
       }
    return a;   
}

Direction Jeu::int2Dir(int a)
{
    Direction d;
    
    switch(a){
       case 0 : d=N;break;
       case 1 : d=S;break;
       case 2 : d=O;break;
       case 3 : d=E;break;
       }
    return d;   
}

void Jeu::ajouteProjectile(int type, Direction dir, int a, int b, int v) {
    Projectile* proj = new Projectile(this, type, dir, a, b, v);
    if (gpProjectile == NULL) gpProjectile = proj;
    else gpProjectile->ajout(proj);
}

void Jeu::ajouteProjectile(int type, Direction dir, int a, int b, int v, int tireur) {
    Projectile* proj = new Projectile(this, type, dir, a, b, v, tireur);
    if (gpProjectile == NULL) gpProjectile = proj;
    else gpProjectile->ajout(proj);
}

void Jeu::ajouteProjectileServ(int type, Direction dir, int a, int b, int v, int tireur) {
    Projectile* proj = new Projectile(this, type, dir, a, b, v, tireur);
    if (gpServeur->getProjectile() == NULL) gpServeur->setProjectile(proj);
    else gpServeur->getProjectile()->ajout(proj);
}

void Jeu::ajouteObjet(int type, int i, int j, int num, int p) {
    Objet* obj = new Objet(this, i, j, type, num, p);
    if (gpObjet == NULL) gpObjet = obj;
    else gpObjet->ajout(obj);
}

void Jeu::ajouteSnipe(int type, double i, double j, double dirx, double diry) {
    Snipe* snp = new Snipe(this, type, i, j, dirx, diry);
    if (gpSnipe == NULL) gpSnipe = snp;
    else gpSnipe->ajout(snp);
}

bool Jeu::isDonjon() {
    return (zone >=12 && zone<=20);
}

bool Jeu::isDehors() {
    return (zone < 12);
}

bool Jeu::isGrotte() {
    return (zone > 21 && zone < 28);
}

void Jeu::lire() {
    int i=0;
    int j=0;
    switch(gpJoueur->getDirection()) {
        case N : i=gpJoueur->getX()+8; j=gpJoueur->getY()+7-2-2; break;
        case S : i=gpJoueur->getX()+8; j=gpJoueur->getY()+24+2; break;
        case O : i=gpJoueur->getX()-1-2; j=gpJoueur->getY()+16; break;
        case E : i=gpJoueur->getX()+16+2; j=gpJoueur->getY()+16; break;
    }
    int id = gpMonde->contenuValue(i, j);
    int val = gpMonde->motifValue(i, j);
    //lit pancarte ou achete
    if (gpMonde->murValue(i, j) && id && gpJoueur->getDirection()==N && 
    (val == 33 || val == 34 || val == 36 || val == 2435 || val == 2437 || val == 2439)) {
        ecrit(id);
        return;
    }
    
    return;

}

void Jeu::trouve(int id) {
    ecrit(id);
    gpJoueur->setTrouve(id);
    gpAudio->playSound(19,2);
}

bool Jeu::finTexte(int id) {
    if (gpJoueur) gpJoueur->setTrouve(0);
    switch (id) {
        case 223 : 
            gpAudio->stopSpecial();
            gpAudio->stopMusic(); 
            gpGenerique->initScore();
            zone = 21;
            gpJoueur->setVie(gpJoueur->getVieMax());
            gpJoueur->setXDep(86);
            gpJoueur->setYDep(24);
            gpJoueur->setDirDep(S);
            gpKeyboard->setMode(14); break;
        case 216 : 
            if(estServeur() && gpServeur->getnombreDeJoueur() > 0) gpPartie->setJeuFini(1);
            else exit(-1);
            break;
        case 225 :  ecrit(227); return true; break;
        case 232 : case 235 : return false; break;
    }
    return false;
}

void Jeu::testRamasse(int x, int y, int w, int h) {
    Objet* gpObjet2 = gpObjet->getSuivant();
    while (gpObjet2 != NULL) {
        if (gpObjet2->getVie()) {
            if (x<gpObjet2->getX()+gpObjet2->getW()-2 && x+w>gpObjet2->getX()+2
            && y<gpObjet2->getY()+gpObjet2->getH()-2 && y+h-4>gpObjet2->getY()+2) {
                gpObjet2->ramasse();
            }
        }
        gpObjet2 = gpObjet2->getSuivant();
    }
}

bool Jeu::testEpee(int x, int y, int w, int h, int type, int force, Direction direction) {
    bool retour = false;
    return retour;
}

bool Jeu::renvoieProj(int i, int j, Direction dir) {
    
    Snipe* gpSnipe2 = gpSnipe->getSuivant();
    while (gpSnipe2 != NULL) {
        if (i >= gpSnipe2->getX() && i < gpSnipe2->getX() + gpSnipe2->getW()
        && j >= gpSnipe2->getY() && j < gpSnipe2->getY() + gpSnipe2->getH()) {
            if (gpSnipe2->renvoit(dir)) {
                gpAudio->playSound(7);
                return true;
            }
        }
        gpSnipe2 = gpSnipe2->getSuivant();
    }
    
    return false;
}

bool Jeu::coupeRideau(int i, int j) {
    if (gpMonde->motifValue(i,j)==2276) {
        if (gpMonde->motifValue(i-16,j)!=2276) gpMonde->passage(i/16,(j/16)-1,0);
        else gpMonde->passage((i/16)-1,(j/16)-1,0);
        return true;
    }
    return false;
}

bool Jeu::tueJoueur(int i, int j, int anim, int numJoueur, Direction direction) {
    
    bool retour = false;
    SDL_Rect* r = gpAutreJoueur[numJoueur]->getZoneBase()->getZone();
    int coupX, coupY, coupW, coupH;
    
    
    switch (direction) {
        case N : coupX=i-15; coupY=j-12; coupW = 35; coupH = 20; break;
        case S : coupX=i-5; coupY=j+19; coupW = 35; coupH = 20; break;
        case O : coupX=i-25; coupY=j; coupW = 25; coupH = 33; break;
        case E : coupX=i+15; coupY=j; coupW = 25; coupH = 33; break;
    }
    
    for (int k = coupX; k < coupX + coupW; k++) {
        for (int l = coupY; l < coupY + coupH; l++) {
            if (k >= 0 && l >=0) {
                  if (k >= gpAutreJoueur[numJoueur]->getX() + r->x &&
                      k < gpAutreJoueur[numJoueur]->getX() + r->x + r->w && 
                      l >= gpAutreJoueur[numJoueur]->getY() + r->y && 
                      l < gpAutreJoueur[numJoueur]->getY() + r->y + r->h)
                    retour = true;
            }
        }
    }
        return retour;        
}

bool Jeu::tueJoueurAvecFleche(int i, int j, int h, int w, int numJoueur) {
    
    bool retour = false;
    SDL_Rect* r = gpAutreJoueur[numJoueur]->getZoneBase()->getZone();
    int coupX, coupY, coupW, coupH;
    
    coupX=i; coupY=j; coupW = h; coupH = w;
    
    for (int k = coupX; k < coupX + coupW; k++) {
        for (int l = coupY; l < coupY + coupH; l++) {
            if (k >= 0 && l >=0) {
                  if (k >= gpAutreJoueur[numJoueur]->getX() + r->x &&
                      k < gpAutreJoueur[numJoueur]->getX() + r->x + r->w && 
                      l >= gpAutreJoueur[numJoueur]->getY() + r->y && 
                      l < gpAutreJoueur[numJoueur]->getY() + r->y + r->h)
                    retour = true;
            }
        }
    }
   
    
        return retour;        
}

bool Jeu::testDegat(int x, int y, int w, int h, int type, int force, Direction direction) {
    if (gpJoueur->getInvincible() || !gpJoueur->getVie()) return false;
    int degats=0;
    SDL_Rect* z = gpJoueur->getZoneBase()->getZone();
    
    int a = z->x+z->w+gpJoueur->getX();
    int b = z->x+gpJoueur->getX();
    int c = z->y+z->h+gpJoueur->getY();
    int d = z->y+gpJoueur->getY();
    
    if (gpJoueur->getTypeAnim() == FLOTTE || gpJoueur->getTypeAnim() == NAGE) c-=4;
    
    if (x < a && x + w > b && y < c && y + h > d) {
        
        if (type!=RETOUR) gpAudio->playSound(33);
        degats=force;
        if (!gpJoueur->getDefense()) degats=degats*2;
        else degats=degats/gpJoueur->getDefense();
        if (type==DEMI) degats=(gpJoueur->getVie()+1)/2;
        if (type!=RETOUR) if (degats < 1) degats = 1;
        gpJoueur->setVie(gpJoueur->getVie()-degats);
        if (type!=RETOUR) gpJoueur->setTypeAnim(TOUCHE);
        if (type == RETOUR) {
            gpAudio->playSound(5);
            gpMonde->transitFull(zone, gpJoueur->getXDep(), gpJoueur->getYDep());
        }
        
        if (type==EXPLOSION) {
            int dfx = (x+(w/2)) - (gpJoueur->getX()+8);
            int dfy = (y+(h/2)) - (gpJoueur->getY()+16);
            if (std::abs(dfx) > std::abs(dfy)) { 
                if (dfx > 0) direction = O; else direction = E;}
            else { if (dfy > 0) direction = N; else direction = S;}
        }
        
        if (type!=RETOUR)
        for (int i = 0; i < 16; i+=2) {
            switch (direction) {
                case N : gpJoueur->setDirection(S); gpJoueur->moveY(-2,2);break;
                case S : gpJoueur->setDirection(N); gpJoueur->moveY(2,2);break;
                case O : gpJoueur->setDirection(E); gpJoueur->moveX(-2,2);break;
                case E : gpJoueur->setDirection(O); gpJoueur->moveX(2,2);break;
            }
        }
        testFin();
        return true;
    }
    
    return false;
}

//a supprimer
bool Jeu::pousse(int i, int j, Direction dir) {
   
    return false;
}

void Jeu::ouvrePorteDonjon(int x, int y) {
}

void Jeu::ouvrePorteMaison(int x, int y) {
}

bool Jeu::isDangereux(int x, int y) {
//supp 20090901
    return false;
}

void Jeu::testInterrupteur(int x, int y) {
//supp 20090901
}

bool Jeu::verifZoneJoueur(int x, int y, int w, int h, int nbdir) {
    //ouvre porte
    ouvrePorteMaison(x+8, y);
    
    //portes verrouilles
    ouvrePorteDonjon(x/*+8*/, y/*-2*/);
    
    testInterrupteur(x+8, y+4);
    
    if (isDangereux(x,y)) return false;
    
    if (!isNotFull(x,y,w,h)) {
        if ((x-(gpJoueur->getX()+gpJoueur->getZoneBase()->getZone()->x)==0 
        || y-(gpJoueur->getY()+gpJoueur->getZoneBase()->getZone()->y)==0) && nbdir==1) {
        //en fonction de la direction
            if (y-(gpJoueur->getY()+gpJoueur->getZoneBase()->getZone()->y)<0 
            && gpJoueur->getDirection()==N) { 
                if ( isMarchable(x-2,y, 1, 1, 0)
                && isMarchable(x-2,y+h-1, 1, 1, 0)
                && isMarchable(x,y, 1, 1, 0)
                && isMarchable(x+7,y, 1, 1, 0)) gpJoueur->moveX(-2);
                if ( isMarchable(x+8,y, 1, 1, 0)
                && isMarchable(x+w-1,y, 1, 1, 0)
                && isMarchable(x+w+1,y, 1, 1, 0)
                && isMarchable(x+w+1,y+h-1, 1, 1, 0)) gpJoueur->moveX(2);
            }
            if (y-(gpJoueur->getY()+gpJoueur->getZoneBase()->getZone()->y)>0 
            && gpJoueur->getDirection()==S) {
                if ( isMarchable(x-2,y, 1, 1, 0)
                && isMarchable(x-2,y+h-1, 1, 1, 0)
                && isMarchable(x,y+h-1, 1, 1, 0)
                && isMarchable(x+7,y+h-1, 1, 1, 0)) gpJoueur->moveX(-2);
                if ( isMarchable(x+8,y+h-1, 1, 1, 0)
                && isMarchable(x+w-1,y+h-1, 1, 1, 0)
                && isMarchable(x+w+1,y, 1, 1, 0)
                && isMarchable(x+w+1,y+h-1, 1, 1, 0)) gpJoueur->moveX(2);
            }
            if (x-(gpJoueur->getX()+gpJoueur->getZoneBase()->getZone()->x)<0 
            && gpJoueur->getDirection()==O) {
                if ( isMarchable(x,y-2, 1, 1, 0)
                && isMarchable(x+w-1,y-2, 1, 1, 0)
                && isMarchable(x,y, 1, 1, 0)
                && isMarchable(x,y+7, 1, 1, 0)) gpJoueur->moveY(-2);
                if ( isMarchable(x,y+8, 1, 1, 0)
                && isMarchable(x,y+h-1, 1, 1, 0)
                && isMarchable(x,y+h+1, 1, 1, 0)
                && isMarchable(x+w+1,y+h+1, 1, 1, 0)) gpJoueur->moveY(2);
            }
            if (x-(gpJoueur->getX()+gpJoueur->getZoneBase()->getZone()->x)>0 
            && gpJoueur->getDirection()==E) {
                if ( isMarchable(x,y-2, 1, 1, 0)
                && isMarchable(x+w-1,y-2, 1, 1, 0)
                && isMarchable(x+w-1,y, 1, 1, 0)
                && isMarchable(x+w-1,y+7, 1, 1, 0)) gpJoueur->moveY(-2);
                if ( isMarchable(x+w-1,y+8, 1, 1, 0)
                && isMarchable(x+w-1,y+h-1, 1, 1, 0)
                && isMarchable(x,y+h+1, 1, 1, 0)
                && isMarchable(x+w+1,y+h+1, 1, 1, 0)) gpJoueur->moveY(2);
            }
        }
        return false;
    }
        
    return true;
}

// renvoit vrai si Link peut y aller
bool Jeu::isNotFull(int x, int y, int w, int h) {
    for (int i = x; i < x + w; i++)
        for (int j = y; j < y + h; j++) {
            if (!isMarchable(i, j, 1, 1, 0)) return false;
        }
    return true;
}

//cas gnral
bool Jeu::isNotFull(int x, int y, int w, int h, int vol, int nage, int spectre) {
    for (int i = x; i < x + w; i++)
        for (int j = y; j < y + h; j++) {
            if (!isMarchable(i, j, 1, 1, spectre)) return false;
        }
    if (isVide(x,y,w,h) && !vol && !spectre) return false;
    if (isWater(x,y,w,h) && !vol && !nage && !spectre) return false;
    return true;
}

//renvoit vrai si plein d'eau, lol
bool Jeu::isWater(int x, int y, int w, int h) {
    for (int i = x; i < x + w; i++)
        for (int j = y; j < y + h; j++) {
            if (i>0 && j>0 && i<gpMonde->regionValue(2) && j<gpMonde->regionValue(3)) 
                if (gpMonde->murValue(i,j)!=EAU) return false;
        }
    return true;
}

//renvoit vrai si plein de vide, super lol
bool Jeu::isVide(int x, int y, int w, int h) {
    for (int i = x; i < x + w; i++)
        for (int j = y; j < y + h; j++) {
            if (i>0 && j>0 && i<gpMonde->regionValue(2) && j<gpMonde->regionValue(3)) 
                if (gpMonde->murValue(i,j)!=VIDE) return false;
        }
    return true;
}

//renvoit vrai prsence de glace
bool Jeu::isGlace(int x, int y, int w, int h) {
    for (int i = x; i < x + w; i++)
        for (int j = y; j < y + h; j++) {
            if (i>0 && j>0 && i<gpMonde->regionValue(2) && j<gpMonde->regionValue(3)) 
                if (gpMonde->murValue(i,j)==GLACE) return true;
        }
    return false;
}

//renvoit vrai si ramassable
bool Jeu::isRamassable(int x, int y, int g) {
    int val = gpMonde->motifValue(x,y);
    if (val>=38 && val <=39 && g==2) return true;
    if (val>=30 && val <=37 && g>=1) return true;
    return false;
}

bool Jeu::isMarchable(int i, int j, int vol, int nage, int spectre) {
    if (i < gpMonde->regionValue(0) || i>= gpMonde->regionValue(2) 
    || j < gpMonde->regionValue(1) || j>= gpMonde->regionValue(3)) return false;
    if (spectre) return true;
   
    int val = gpMonde->murValue(i, j);
    if (val == LIBRE || val == GLACE) return true;
    if (val == VIDE && vol) return true;
    if (val == EAU && (vol || nage)) return true;
    return false;
}

bool Jeu::isLink(int x, int y, int w, int h) {
    SDL_Rect* r = gpJoueur->getZoneBase()->getZone();
    int x1 = gpJoueur->getX();
    int y1 = gpJoueur->getY();
    
    if (gpJoueur->getVie()>0) {
        if ((x1 + r->x + r->w > x && x1 + r->x < x+w)
        && (y1 + r->y + r->h > y && y1 + r->y < y+h)) 
            return true;
    }
    return false;
}

//ramasse objet rammassable (pancarte, pierre, etc.)
void Jeu::ramasse() {
    int i;
    int j;
    int g = gpJoueur->hasObjet(O_GANTS);
    switch(gpJoueur->getDirection()) {
        case N : i=gpJoueur->getX()+8; j=gpJoueur->getY()+7-2; break;
        case S : i=gpJoueur->getX()+8; j=gpJoueur->getY()+24+2; break;
        case O : i=gpJoueur->getX()-1-2; j=gpJoueur->getY()+16; break;
        case E : i=gpJoueur->getX()+16+2; j=gpJoueur->getY()+16; break;
    }
    if (isRamassable(i,j,g)) {gpJoueur->setTypeAnim(SOULEVE); return;}
    if (g < 2) if (isRamassable(i,j,2)) gpJoueur->setTypeAnim(LOURD);
}

int Jeu::enleve() {
    int i;
    int j;
    int g = gpJoueur->hasObjet(O_GANTS);
    switch(gpJoueur->getDirection()) {
        case N : i=gpJoueur->getX()+8; j=gpJoueur->getY()+7-2; break;
        case S : i=gpJoueur->getX()+8; j=gpJoueur->getY()+24+2; break;
        case O : i=gpJoueur->getX()-1-2; j=gpJoueur->getY()+16; break;
        case E : i=gpJoueur->getX()+16+2; j=gpJoueur->getY()+16; break;
    }
    if (!isRamassable(i,j, g)) return 0;
    int val = gpMonde->motifValue(i,j);
    int new_val = val+9;
    int obj=0;
    
    switch(val) {
        case 30 : obj = 1; new_val = 40; break;
        case 31 : obj = 2; new_val = 40; break;
        case 32 : obj = 3; new_val = 41; break;
        case 33 : obj = 4; new_val = 42; break;
        case 34 : obj = 4; new_val = 43; break;
        case 35 : obj = 3; new_val = 44; if (zone == 5) new_val = 46; break;
        case 36 : obj = 4; new_val = 45; if (zone == 5) new_val = 47; break;
        case 37 : obj = 5; new_val = 44; break;
        case 38 : obj = 6; new_val = 41; break;
        case 39 : obj = 6; new_val = 44; if (zone == 5) new_val = 46; break;
    }
    gpMonde->setValeur(i,j,new_val,0);
    if (isDonjon()) {
        if(gpMonde->contenuValue(i, j)==51) gpMonde->setValeur(i,j,1760,-1,0,51);
        if(gpMonde->contenuValue(i, j)==50) gpMonde->setValeur(i,j,1759,-1,0,50);
    }
    
    gpAudio->playSound(34);
    
    return obj;
}

void Jeu::setMenu(bool b) {
    menu = b;
    if (!menu) gpMenu->menuOut();
    else {gpMenu->menuIn();}
}

void Jeu::ecrit(int id, bool anim, bool cadre, int x, int y, int w, int h) {
    setText(true);
    gpTexte->setTexte(id, x, y, w, h, anim, cadre);
}

void Jeu::affiche(SDL_Surface* gpScreen, std::string s, int x, int y) {
    gpTexte->affiche(gpScreen, s, x, y);
}

void Jeu::setStop(bool b) { stop = b;}

void Jeu::setTransition(bool b) { transition = b;}

void Jeu::setText(bool b) { text = b; /*stop = text;*/}

void Jeu::setPhg(int i, int j) {phg[0]=i;phg[1]=j;}

void Jeu::setZone(int i) {zone=i;}

bool Jeu::getStop() { return stop;}
bool Jeu::getTransition() { return transition;}
int Jeu::getZone() { return zone;}
int Jeu::getPhg(int i) { return phg[i];}
bool Jeu::getMenu() { return menu;}
bool Jeu::getText() { return text;}
SDL_Surface* Jeu::getImageObjets() { return imageObjets;}

int Jeu::getVueHorz() {return vueHorz;}
int Jeu::getVueVert() {return vueVert;}
void Jeu::setVueHorz(int i) {vueHorz = i;}
void Jeu::setVueVert(int i) {vueVert = i;}

void Jeu::setNewZone(int nz) { newzone = nz;}
void Jeu::setNewX(int nx) { newx = nx;}
void Jeu::setNewY(int ny) { newy = ny;}


void Jeu::finZone() {
    gpJoueur->setX(newx);
    gpJoueur->setY(newy);
    gpJoueur->saveSemiEmpl();
    
    if (zone != newzone) {
        gpJoueur->saveEmpl();
        videListes();
        zone = newzone;
        if (isDonjon() && !gpJoueur->getCharge() && (gpMonde->regionValue(1) || zone!=20)) 
            gpJoueur->setDirection(N);
        if (zone==4 && newx==16*42 && newy==16*5-8) gpJoueur->setDirection(E);
        gpMonde->changeZone(zone);
        
        gpAudio->playMusic(zone);
    } else gpMonde->detRegion(zone);
    
    if (gpJoueur->getTypeAnim()==EMMENE) gpJoueur->setTypeAnim(MARCHE);
    gpJoueur->visible();
    scrolling();
}

void Jeu::testFin() {
     
    int depart=0;
    int posX,posY;
    if (transition) return;
    //si peut ramasser un objet
    testRamasse(gpJoueur->getX(), gpJoueur->getY()+8, 16, 16);
    //si dans l'eau
    if (isWater(gpJoueur->getX(), gpJoueur->getY()+8, 16,16)) {
        gpJoueur->bigPlouf();
    }else {
        gpJoueur->antiPlouf();
        if (gpJoueur->getTypeAnim()==FLOTTE || gpJoueur->getTypeAnim()==NAGE) 
            gpJoueur->setTypeAnim(AUCUNE);
    }
    //si tombe dans le vide
    if (isVide(gpJoueur->getX(), gpJoueur->getY()+8, 16,16)) {
        gpAudio->playSound(12);
        gpJoueur->setTypeAnim(CHUTE);
    }
    switch(zone) {
        case 21 : //SALLE DEBUT JEU !!
            if (gpJoueur->getY()>240-24-4)
               {
                  gpPartie->positionRevie(posX,posY);
                  gpMonde->transitFull(gpPartie->getAreneToZone(), posX, posY);
               }
            break;
    }
    
    if ((isDonjon() || isGrotte()) && !transition) {
        
        int dir=-1;
        
        if (gpJoueur->getY()<gpMonde->regionValue(1)+6 && !transition) {
            gpMonde->transitSalle(N);dir=N;}
        if (gpJoueur->getY()>gpMonde->regionValue(3)-28 && !transition) {
            gpMonde->transitSalle(S);dir=S;}
        if (gpJoueur->getX()<gpMonde->regionValue(0)+6 && !transition) {
            gpMonde->transitSalle(O);dir=O;}
        if (gpJoueur->getX()>gpMonde->regionValue(2)-24 && !transition) {
            gpMonde->transitSalle(E);dir=E;}
            
    }
}

void Jeu::finDePartie()
{
     gpKeyboard->setMode(11);
}
